/**********************************************************************************
File Name      : a_compress.c
Module Name    : TRANS Library
Description    : This file contains a function to perform a-law compression.
					Copyright(c) 2000 Analog Devices Inc.    
************************************************************************************/

//#define CID_SIMULATION

#ifndef CID_SIMULATION
#include <oswrapper.h>
#include <CidTxCon.h>
#include <CidTxProto.h>
#else
#include <math.h>
#include "typedef.h"
#include "CidTxCon.h"
#include "CidTxProto.h"
#endif


#define BIAS 33
#define CLIP1 8158
#define CLIP 8191

#if 1 //def CALLER_IDTx
/****************************************************************************

  Func name   : mu_compress

  Purpose     : mu-law compression
  Description : This routine determines the 8-bit (right-justified) output of 
                mu-law
                compression
  Domain      : (Input) -8192 .. +8191,   (Output) 0 .. 255

*****************************************************************************/

void
mu_compress(
    Word16 in[], /* (i) - {Pointer to the input sequence } */
    UCHAR  out[]      /* (o) - {Pointer to the output sequence} */
    
)
{
    Word32 i;
    Word16 data, sign;
    Word16 chord, step;
    static Word16 exp_lut[256]={0,0,1,1,2,2,2,2,3,3,3,3,3,3,3,3,
                             4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
                             5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                             5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7};

    /* { Loop until all the input data processed } */
    for (i = 0; i < 80; i++)
    {
        /* { Read the input data sequentially } */
        data = in[i];

        /* { Check the sign of input data, and get the magnitude of input data } */
        sign = (data >> 8) & 0x80;
        if (sign != 0)
        {
            data = -data;
        }
        /* {Limit the magnitude of input data in the range of 0 to 8158} */
        if (data > CLIP1)
        {
            data = CLIP1;
        }

        /* { Add the bias to the input data } */
        data = data + BIAS;

        /* { Convert 14 bits data to 8 bit data according to linear mu-law } */
        chord = exp_lut[(data >> 5)];
        step = (data >> (chord +1)) & 0x0F;

        /* { Invert all the bits } */
        out[i] = (UCHAR) (255-(sign | (chord << 4) | step));
    }
}

/****************************************************************************
  Func name   : a_compress
  Purpose     : a-law compression
  Description : This routine determines the 8-bit (right-justified) output of 
                A-law
                compression
  Domain      : (Input) -8192 .. +8191,   (Output) 0 .. 255
  
 ****************************************************************************/

void a_compress(
    Word16 in[], /* (i) - {Pointer to the input sequence } */
    UCHAR  out[]      /* (o) - {Pointer to the output sequence} */
    
)
{
    Word32 i;
    Word16 data, sign;
    Word16 chord, step;
    static Word16 exp_lut[128]={0,1,2,2,3,3,3,3,4,4,4,4,4,4,4,4,
                             5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7};

    /* { Loop until all the input data processed } */
    for (i = 0; i < 80; i++)
    {
        /* { Read the input data sequentially } */
        data = in[i];

        /* { Check the sign of input data, and get the magnitude of input data }*/
         sign = (data >> 8) & 0x80;
        if (sign != 0)
        {
            data = -data;
        }

        /* {Limit the magnitude of input data in the range of 0 to 8191} */
        if (data > CLIP)
        {
            data = CLIP;
        }
        /* { Convert the 14 bit number to 8 bit number using linear A-law } */

        chord = exp_lut[(data >> 6)];

        if (chord == 0)
        {
            step = (data >> 2) & 0x0F;
        }
        else
        {
            step = (data >>  (chord + 1) ) & 0x0F;
        }

        out[i] =  (UCHAR)((sign | (chord << 4) | step)^0x55);
    }

}

void
mu_compress_short(
    Word16 in[], /* (i) - {Pointer to the input sequence } */
    Word16 out[]      /* (o) - {Pointer to the output sequence} */

)
{
    Word32 i;
    Word16 data, sign;
    Word16 chord, step;
    static Word16 exp_lut[256]={0,0,1,1,2,2,2,2,3,3,3,3,3,3,3,3,
                             4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
                             5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                             5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7};

    /* { Loop until all the input data processed } */
    for (i = 0; i < 80; i++)
    {
        /* { Read the input data sequentially } */
        data = in[i];

        /* { Check the sign of input data, and get the magnitude of input data }
 */
        sign = (data >> 8) & 0x80;
        if (sign != 0)
        {
            data = -data;
        }
        /* {Limit the magnitude of input data in the range of 0 to 8158} */
        if (data > CLIP1)
        {
            data = CLIP1;
        }

        /* { Add the bias to the input data } */
        data = data + BIAS;

        /* { Convert 14 bits data to 8 bit data according to linear mu-law } */
        chord = exp_lut[(data >> 5)];
        step = (data >> (chord +1)) & 0x0F;

        /* { Invert all the bits } */
        out[i] = 255-(sign | (chord << 4) | step);
    }
}


/****************************************************************************

  Func name   :  a_expand

  Purpose     : a-law expansion
  Description : This function determines the 14-bit linear PCM value (right-justified)
                from the 8-bit (right justified) log (a-law) value
  Domain      : (Input) 0 .. 255,   (Output) -8192 .. +8191


*****************************************************************************/

void
a_expand(
    UCHAR in[],/* (i) - {Pointer to the input sequence } */
    Word16 out[]     /* (o) - {Pointer to the output sequence} */
    //Word32 n           /* (i) - {The number of input sequence } */
)
{
    Word32 i;
    Word16 data, sign, expand;
    Word16 chord, step;

    /* { Loop until all the input data processed } */
    for (i = 0; i < 80; i++)
    {
        /* { Read the input data } */
        data =(Word16) in[i];
        data =  data ^ 0x55;
        /* { Get the sign, chord and step bits from the input data }*/
        sign = (data >> 7);
        chord = ((data & 0x70) >> 4);
        step = (data & 0x0F);

        /* { Convert the 8 bits input to 14 bits output data } */
        if (chord == 0)
        {
            expand = (step << 2) + 1 ;
        }
        else
        {
            step = (step << 2) + 65;
            expand = step << (chord-1);
        }

        if( sign == 1)
        {
            expand = - expand;
        }
        out[i] = expand;
    }
}

/****************************************************************************

  Func name   :  mu_expand

  Purpose     : mu-law expansion
  Description : This function determines the 14-bit linear PCM value (right-justified)
                from the 8-bit (right justified) log (mu-law) value
  Domain      : (Input) 0 .. 255,   (Output) -8192 .. +8191

****************************************************************************/

void
mu_expand(
    UCHAR in[], /* sneha(i) - {Pointer to the input sequence } */
    Word16 out[]            /* (o) - {Pointer to the output sequence} */
    //Word32 n                 /* (i) - {The number of input sequence  } */
)
{
    Word32 i;
    Word16 data, sign, expand;
    Word16 chord, step;

    /* { Loop until all the input data are processed } */
    for (i = 0; i < 80; i++)
    {
        /* { Read the input data } */
        data = (Word16) in[i];
        data = 255 - data;

         /* { Get the sign, chord and step bits from the input data }*/
        sign = (data >> 7);
        chord = ((data & 0x70) >> 4);
        step = (data & 0x0F);

        /* { Convert the 8 bits input to 14 bits output data } */
        step = (step <<1) + 33;
        expand = step << chord;
        expand = expand - 33;

        if( sign == 1)
        {
            expand = - expand;
        }
        out[i] = expand;
    }
}

#endif
